<?php

namespace Drupal\themespace\Plugin\Discovery;

use Drupal\Core\Plugin\Discovery\ContainerDerivativeDiscoveryDecorator;

/**
 * Plugin discovery decorator which adds derivative definitions.
 *
 * This version of the deriver discovery decorator just implements the
 * \Drupal\themespace\Plugin\Discovery\ProviderTypedDiscoveryInterface by adding
 * the ::getModuleDefinitions() and ::getThemeDefinition() methods.
 */
class ProviderTypedDeriverDiscoveryDecorator extends ContainerDerivativeDiscoveryDecorator implements ProviderTypedDiscoveryInterface {

  /**
   * {@inheritdoc}
   */
  public function getModuleDefinitions(): array {
    if ($this->decorated instanceof ProviderTypedDiscoveryInterface) {
      $pluginDefs = $this->decorated->getModuleDefinitions();
      return $this->getDerivatives($pluginDefs);
    }

    // If decorated is not provider type aware, treat all plugin definitions
    // as if they are "module" provided definitions, getDefinitions() method.
    return parent::getDefinitions();
  }

  /**
   * {@inheritdoc}
   */
  public function getThemeDefinitions(): array {
    if ($this->decorated instanceof ProviderTypedDiscoveryInterface) {
      $pluginDefs = $this->decorated->getThemeDefinitions();
      return $this->getDerivatives($pluginDefs);
    }

    // If decorated discovery is not provider type aware, treat all definitions
    // as if they are "module" provided definitions - so there would be no
    // theme definition to create derivatives for.
    return [];
  }

}
